package com.jiuqi.util;

package .tmp;

import com.jiuqi.util.SparseCell;
import java.util.ArrayList;

public class SparseMatrix {
  private ArrayList cells = new ArrayList();
  
  protected int find(int col, int row) {
    int sb = 0, se = this.cells.size() - 1, flag = -1;
    while (sb <= se) {
      int sp = sb + se >> 1;
      SparseCell cell = this.cells.get(sp);
      flag = cell.compareTo(col, row);
      if (flag < 0) {
        sb = sp + 1;
        continue;
      } 
      if (flag > 0) {
        se = sp - 1;
        continue;
      } 
      sb = sp;
      break;
    } 
    return (flag == 0) ? sb : (-sb - 1);
  }
  
  public void clear() {
    this.cells.clear();
  }
  
  public int size() {
    return this.cells.size();
  }
  
  public Object get(int index) {
    return ((SparseCell)this.cells.get(index)).data;
  }
  
  public int getCol(int index) {
    return ((SparseCell)this.cells.get(index)).col;
  }
  
  public int getRow(int index) {
    return ((SparseCell)this.cells.get(index)).row;
  }
  
  public Object get(int col, int row) {
    int index = find(col, row);
    if (index >= 0)
      return ((SparseCell)this.cells.get(index)).data; 
    return null;
  }
  
  public void put(int col, int row, Object data) {
    int index = find(col, row);
    if (index >= 0) {
      if (data == null) {
        this.cells.remove(index);
      } else {
        ((SparseCell)this.cells.get(index)).data = data;
      } 
    } else if (data != null) {
      index = -1 - index;
      SparseCell cell = new SparseCell();
      cell.col = col;
      cell.row = row;
      cell.data = data;
      this.cells.add(index, cell);
    } 
  }
  
  public void colinsert(int index) {
    for (int i = this.cells.size() - 1; i >= 0; i--) {
      SparseCell cell = this.cells.get(i);
      if (cell.col >= index)
        cell.col++; 
    } 
  }
  
  public void coldelete(int index) {
    for (int i = this.cells.size() - 1; i >= 0; i--) {
      SparseCell cell = this.cells.get(i);
      if (cell.col == index) {
        this.cells.remove(i);
      } else if (cell.col > index) {
        cell.col--;
      } 
    } 
  }
  
  public void colcountchange(int colcount) {
    for (int i = this.cells.size() - 1; i >= 0; i--) {
      SparseCell cell = this.cells.get(i);
      if (cell.col >= colcount)
        this.cells.remove(i); 
    } 
  }
  
  public void rowinsert(int index) {
    for (int i = this.cells.size() - 1; i >= 0; i--) {
      SparseCell cell = this.cells.get(i);
      if (cell.row >= index)
        cell.row++; 
    } 
  }
  
  public void rowdelete(int index) {
    for (int i = this.cells.size() - 1; i >= 0; i--) {
      SparseCell cell = this.cells.get(i);
      if (cell.row == index) {
        this.cells.remove(i);
      } else if (cell.row > index) {
        cell.row--;
      } 
    } 
  }
  
  public void rowcountchange(int rowcount) {
    for (int i = this.cells.size() - 1; i >= 0; i--) {
      SparseCell cell = this.cells.get(i);
      if (cell.row >= rowcount)
        this.cells.remove(i); 
    } 
  }
}
